"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var UserModel_1 = require("../../models/UserModel");
var $ = require('jquery/dist/jquery.min');
var User = (function (_super) {
    __extends(User, _super);
    function User(user, userFactory) {
        _super.call(this);
        this.userFactory = userFactory;
        this.passwordHashed = false;
        if (!user)
            return;
        var keys = Object.keys(this);
        for (var i = 0; i < keys.length; ++i) {
            if (user[keys[i]] !== undefined && (typeof user[keys[i]]) !== 'function')
                this[keys[i]] = user[keys[i]];
        }
        this.hashPassword();
    }
    User.prototype.LoginModel = function () {
        return { username: this.username, password: this.password };
    };
    ;
    User.prototype.hashPassword = function () {
        if (this.passwordHashed || !this.password || this.password === '') {
            this.passwordHashed = true;
            return this.password;
        }
        this.password = this.userFactory.HashString(this.password);
        this.passwordHashed = true;
    };
    User.prototype.toJson = function () {
        return JSON.stringify(this);
    };
    User.prototype.getBland = function () {
        return {
            _id: this._id,
            username: this.username,
            password: this.password,
            first_name: this.first_name,
            last_name: this.last_name,
            email: this.email,
            role: this.role
        };
    };
    return User;
}(UserModel_1.UserModel));
exports.User = User;
var UserFactory = (function () {
    function UserFactory(bcrypt, $http) {
        this.bcrypt = bcrypt;
        this.$http = $http;
        this.LoggedIn = false;
        this.LoadCredentials();
        UserFactory.initialized = true;
    }
    UserFactory.prototype.Initialize = function () {
        if (!UserFactory.initialized || !this.user)
            this.LoadCredentials();
    };
    UserFactory.prototype.getSelectList = function (resolve, reject) {
        this.httpServerCall('/api/user/', 'get', undefined, function (data) {
            var list = data.data;
            var resultList = [];
            for (var i = 0; i < list.length; i++) {
                resultList.push(new Object({ label: list[i].first_name, value: list[i]._id }));
            }
            resolve(resultList);
        }, function (error) {
            reject(error);
        });
    };
    UserFactory.prototype.getOneById = function (id, resolve, reject) {
        this.httpServerCall('/api/user/' + id, 'get', undefined, function (data) {
            resolve(data.data);
        }, function (error) {
            reject(error);
        });
    };
    UserFactory.prototype.StoreCredentials = function (user, token) {
        if (!user || !token)
            return;
        this.user = user;
        this.token = token;
        this.LoggedIn = true;
        user.password = '';
        if (user['userFactory'] != undefined)
            user['userFactory'] = undefined;
        localStorage.setItem('user', angular.toJson(user));
        localStorage.setItem('token', token);
    };
    UserFactory.prototype.ClearCredentials = function () {
        this.user = undefined;
        this.token = undefined;
        this.LoggedIn = false;
        localStorage.removeItem('token');
        localStorage.removeItem('user');
    };
    UserFactory.prototype.LoadCredentials = function () {
        var token = localStorage.getItem('token');
        var user = undefined;
        if (localStorage.getItem('user') !== undefined)
            user = angular.fromJson(localStorage.getItem('user'));
        if (!token || !user || !user.username)
            return this.LoggedIn = false;
        this.token = token;
        this.setUser(user);
        this.LoggedIn = true;
    };
    UserFactory.prototype.setUser = function (user) {
        return this.user = user;
    };
    UserFactory.prototype.httpServerCall = function (url, method, data, success, failure) {
        if (failure === void 0) { failure = undefined; }
        url = url.replace('//', '/');
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        if (this.token) {
            headers['Authorization'] = 'Bearer ' + this.token;
        }
        var options = {
            url: url,
            method: method,
            headers: headers
        };
        if (data) {
            if (options.method.toUpperCase() === 'GET') {
                options['params'] = data;
            }
            else {
                options['data'] = $.param(data);
            }
        }
        this.$http(options).then(function (response) {
            success && success(response);
        }, function (error) {
            failure && failure(error);
        });
    };
    UserFactory.prototype.ChangePassword = function (pswd, newPassword, success, failure) {
        if (failure === void 0) { failure = undefined; }
        this.httpServerCall('/api/user/ChangePassword', 'put', {
            username: this.user.username,
            old_password: pswd,
            password: newPassword
        }, function (response) {
            if (!response || !response.data)
                return failure('Server error.');
            if (response.data.error)
                return failure(response.data.message);
            if (response.data.success)
                return success();
            failure("not a success");
        }, failure);
    };
    UserFactory.prototype.HashString = function (toHash) {
        if (this.bcrypt === undefined)
            return toHash;
        return this.bcrypt.hashSync(toHash, UserModel_1.ClientSalt);
    };
    UserFactory.prototype.$get = function () {
        return this;
    };
    UserFactory.Factory = function () {
        return ['bcrypt', '$http', function (bcrypt, http) {
                return new UserFactory(bcrypt, http);
            }];
    };
    UserFactory.factoryName = 'UserFactory';
    UserFactory.$inject = ['bcrypt', '$http'];
    UserFactory.initialized = false;
    return UserFactory;
}());
exports.UserFactory = UserFactory;
//# sourceMappingURL=factory.js.map